package com.retail_store.order_management.utils;

import com.retail_store.order_management.application.OrderManagement;
import com.retail_store.order_management.constants.Category;
import com.retail_store.order_management.constants.Color;
import com.retail_store.order_management.constants.Gender;
import com.retail_store.order_management.constants.Size;
import com.retail_store.order_management.model.Order;
import com.retail_store.order_management.service.RetailStore;

import java.util.*;
import java.util.logging.Logger;

public final class Utility {
    private static final Scanner SCANNER = null;
    private static final Logger LOGGER = null;

    private Utility() {
    }

    public static Category getCategory(boolean check, String message) {
        String categoryOptions = Arrays.stream(Category.values()).filter(category -> !category.equals(Category.EXIT)).map(category -> "(" + category.getOption() + ")" + category).reduce((categories, category) -> categories + " " + category).get() + "\nPress 0 to go back.";
        if (check) {
            getLogger().info("\n------------------------\n" + message + "\n------------------------\n" + categoryOptions + "\n------------------------\n");
        }
        int option = getInt();
        return Arrays.stream(Category.values()).filter(category -> category.getOption() == option).findFirst().orElseGet(() -> {
            getLogger().info("\n------------------------\n+Please enter valid category\n------------------------\n" + categoryOptions + "\n------------------------\n");
            return getCategory(false, message);
        });
    }

    public static Size getSize(boolean check, String message) {
        String categoryOptions = Arrays.stream(Size.values()).filter(size -> !size.equals(Size.EXIT)).map(size -> "(" + size.getOption() + ")" + size).reduce((sizes, size) -> sizes + " " + size).get() + "\nPress 0 to go back.";
        if (check) {
            getLogger().info("\n------------------------\n" + message + "\n------------------------\n" + categoryOptions + "\n------------------------\n");
        }
        int option = getInt();
        return Arrays.stream(Size.values()).filter(size -> size.getOption() == option).findFirst().orElseGet(() -> {
            getLogger().info("\n------------------------\nPlease enter valid category\n------------------------\n" + categoryOptions + "\n------------------------\n");
            return getSize(false, message);
        });
    }

    public static Gender getGender(boolean check, String message) {
        String genderOptions = Arrays.stream(Gender.values()).filter(gender -> !gender.equals(Gender.EXIT)).map(gender -> "(" + gender.getOption() + ")" + gender).reduce((genders, gender) -> genders + " " + gender).get() + "\nPress 0 to go back.";
        if (check) {
            getLogger().info("\n------------------------\n" + message + "\n------------------------\n" + genderOptions + "\n------------------------\n");
        }
        int option = getInt();
        return Arrays.stream(Gender.values()).filter(gender -> gender.getOption() == option).findFirst().orElseGet(() -> {
            getLogger().info("\n------------------------\nPlease enter valid gender\n------------------------\n" + genderOptions + "\n------------------------\n");
            return getGender(false, message);
        });
    }

    public static Color getColor(boolean check, String message) {
        String colorOptions = Arrays.stream(Color.values()).filter(color -> !color.equals(Color.EXIT)).map(color -> "(" + color.getOption() + ")" + color).reduce((categories, category) -> categories + " " + category).get() + "\nPress 0 to go back.";
        if (check) {
            getLogger().info("\n------------------------\n" + message + "\n------------------------\n" + colorOptions + "\n------------------------\n");
        }
        int option = getInt();
        return Arrays.stream(Color.values()).filter(color -> color.getOption() == option).findFirst().orElseGet(() -> {
            getLogger().info("\n------------------------\n+Please enter valid color\n------------------------\n" + colorOptions + "\n------------------------\n");
            return getColor(false, message);
        });
    }

    public static double getDouble(boolean flag, String message) {
        double number;
        Scanner scanner = new Scanner(System.in);
        if (flag) {
            getLogger().info(message);
        }
        try {
            number = scanner.nextDouble();
            if (number < 0) {
                throw new RuntimeException();
            }
            return number;
        } catch (InputMismatchException e) {
            getLogger().warning("\n------------------------\nPlease enter valid price\n------------------------\n");
            scanner.reset();
            return getDouble(false, message);
        }
    }

    public static int getInt() {
        int number;
        Scanner scanner = new Scanner(System.in);
        try {
            number = scanner.nextInt();
            if (number < 0) {
                throw new RuntimeException();
            }
            return number;
        } catch (Exception e) {
            getLogger().warning("\n------------------------\nPlease enter valid number\n------------------------\n");
            scanner.reset();
            return getInt();
        }
    }

    public static Logger getLogger() {
        return LOGGER == null ? Logger.getLogger(OrderManagement.class.getName()) : LOGGER;
    }

    public static Scanner getScanner() {
        return SCANNER == null ? new Scanner(System.in) : SCANNER;
    }

    public static LinkedHashMap<Size, Integer> getProductSizes() {
        LinkedHashMap<Size, Integer> productSizes = new LinkedHashMap<>();
        getLogger().info("Please enter the available stock for particular sizes \n Number of products with size S");
        productSizes.put(Size.S, getInt());
        getLogger().info("Number of products with size M");
        productSizes.put(Size.M, getInt());
        getLogger().info("Number of products with size L");
        productSizes.put(Size.L, getInt());
        getLogger().info("Number of products with size XL");
        productSizes.put(Size.XL, getInt());
        getLogger().info("Number of products with size XXL");
        productSizes.put(Size.XXL, getInt());
        return productSizes;
    }

    public static Map<Size, Integer> getProductSizes(int s, int m, int l, int xl, int xxl) {
        LinkedHashMap<Size, Integer> productSizes = new LinkedHashMap<>();
        productSizes.put(Size.S, s);
        productSizes.put(Size.M, m);
        productSizes.put(Size.L, l);
        productSizes.put(Size.XL, xl);
        productSizes.put(Size.XXL, xxl);
        return productSizes;
    }

    public static String productIdGenerator() {
        return Arrays.stream(UUID.randomUUID().toString().split("-")).skip(1).findFirst().get() + "#" + RetailStore.class.getSimpleName();
    }

    public static String orderIdGenerator() {
        return Arrays.stream(UUID.randomUUID().toString().split("-")).findFirst().get() + "#" + Order.class.getSimpleName();
    }
}